/*
 * sewtup.c: The setup for the Atmolight-plugin
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
#include "setup.h"

#define SETUP_SLEEPTIME      "Sleeptime"
#define SETUP_PERCENTNEW     "Percentage new Color"
#define SETUP_SATOFFSET      "Saturation Offset"
#define SETUP_DARKLIMIT      "Darkness Limit"
#define SETUP_MODE           "Mode"
#define SETUP_STATICRED      "Red"
#define SETUP_STATICGREEN    "Green"
#define SETUP_STATICBLUE     "Blue"
#define SETUP_CALIBRED       "Red"
#define SETUP_CALIBGREEN     "Green"
#define SETUP_CALIBBLUE      "Blue"
#define SETUP_RAWRED         "Red"
#define SETUP_RAWGREEN       "Green"
#define SETUP_RAWBLUE        "Blue"

// seconds the next timer must be in the future to assume manual start
#define MANUALSTART       600

static const char *useModeTexts[4];

// --- cAtmoSetup ------------------------------------------------------------

cAtmoSetup::cAtmoSetup()
{
  useModeTexts[0] = "live";              // generate colors out of live picture
  useModeTexts[1] = "static colors";     // generate static color
  useModeTexts[2] = "white calibration"; // user can calibrate generated color to white
  useModeTexts[3] = "raw PWM";           // send pwm values directly (0-255)

  // assumption of manual/automatic start of vdr is done in GetStartMode(),
  // because timers are not loaded yet

  SleepTime    = 40;      // milli-seconds
  PercentNew   = 50;      // percentage of new color (for filtering)
  SatOffset    = 0;       // saturation offset
  DarkLimit    = 0;       // HSV.V < limit -> HSV.V=0 (no light)
  Mode         = 0;       // atmolight-mode
  StaticRed    = 0;       // static mode: red
  StaticGreen  = 0;       // static mode: green
  StaticBlue   = 0;       // static mode: blue
  CalibRed     = 31;      // calibration mode: red
  CalibGreen   = 31;      // calibration mode: green
  CalibBlue    = 31;      // calibration mode: blue
  RawRed       = 31;      // raw mode: red
  RawGreen     = 31;      // raw mode: green
  RawBlue      = 31;      // raw mode: blue
}

bool cAtmoSetup::SetupParse( const char * Name, const char * Value )
{
       if (!strcasecmp(Name, SETUP_SLEEPTIME))   SleepTime   = atoi(Value);
  else if (!strcasecmp(Name, SETUP_PERCENTNEW))  PercentNew  = atoi(Value);
  else if (!strcasecmp(Name, SETUP_SATOFFSET))   SatOffset   = atoi(Value);
  else if (!strcasecmp(Name, SETUP_DARKLIMIT))   DarkLimit   = atoi(Value);
  else if (!strcasecmp(Name, SETUP_MODE))        Mode        = atoi(Value);
  else if (!strcasecmp(Name, SETUP_STATICRED))   StaticRed   = atoi(Value);
  else if (!strcasecmp(Name, SETUP_STATICGREEN)) StaticGreen = atoi(Value);
  else if (!strcasecmp(Name, SETUP_STATICBLUE))  StaticBlue  = atoi(Value);
  else if (!strcasecmp(Name, SETUP_CALIBRED))    CalibRed    = atoi(Value);
  else if (!strcasecmp(Name, SETUP_CALIBGREEN))  CalibGreen  = atoi(Value);
  else if (!strcasecmp(Name, SETUP_CALIBBLUE))   CalibBlue   = atoi(Value);
  else if (!strcasecmp(Name, SETUP_RAWRED))      RawRed      = atoi(Value);
  else if (!strcasecmp(Name, SETUP_RAWGREEN))    RawGreen    = atoi(Value);
  else if (!strcasecmp(Name, SETUP_RAWBLUE))     RawBlue     = atoi(Value);
  else return false;

  return true;
}

// assumption of manual/automatic start of vdr
void cAtmoSetup::GetStartMode(void)
{
 time_t Now = time(NULL);
 cTimer *timer = Timers.GetNextActiveTimer();
 time_t Next  = timer ? timer->StartTime() : 0;
 time_t Delta = timer ? Next - Now : 0;

 if (!timer || Delta > MANUALSTART)
 { // Apparently the user started VDR manually                      
   dsyslog("Atmolight: assuming manual start of VDR");
   AtmoSetup.atmo_on = true;
 } else {
   dsyslog("Atmolight: assuming automatic start of VDR");
   AtmoSetup.atmo_on = false;
 }
}

// --- cMenuSetupAtmo --------------------------------------------------------

cMenuSetupAtmo::cMenuSetupAtmo(void)
{
  Set();
}

void cMenuSetupAtmo::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditIntItem("Sleeptime (ms)", &AtmoSetup.SleepTime, 40, 1000)); // 40ms = 25fps
  Add(new cMenuEditIntItem("Percentage new Color", &AtmoSetup.PercentNew, 1, 100));
  Add(new cMenuEditIntItem("Saturation Offset", &AtmoSetup.SatOffset, 0, 20));
  Add(new cMenuEditIntItem("Darkness Limit", &AtmoSetup.DarkLimit, 0, 10));
  Add(new cMenuEditStraItem("Mode", &AtmoSetup.Mode, 4, useModeTexts));

  if (AtmoSetup.Mode == 1)
  {
    Add(new cMenuEditIntItem("Red"  , &AtmoSetup.StaticRed, 0, 31));
    Add(new cMenuEditIntItem("Green", &AtmoSetup.StaticGreen, 0, 31));
    Add(new cMenuEditIntItem("Blue" , &AtmoSetup.StaticBlue, 0, 31));
  }

  if (AtmoSetup.Mode == 2)
  {
    Add(new cMenuEditIntItem("Red"  , &AtmoSetup.CalibRed, 0, 31));
    Add(new cMenuEditIntItem("Green", &AtmoSetup.CalibGreen, 0, 31));
    Add(new cMenuEditIntItem("Blue" , &AtmoSetup.CalibBlue, 0, 31));
  }

  if (AtmoSetup.Mode == 3)
  {
    Add(new cMenuEditIntItem("Red"  , &AtmoSetup.RawRed, 0, 255));
    Add(new cMenuEditIntItem("Green", &AtmoSetup.RawGreen, 0, 255));
    Add(new cMenuEditIntItem("Blue" , &AtmoSetup.RawBlue, 0, 255));
  }

  SetCurrent(Get(current));
  Display();
}

void cMenuSetupAtmo::Store(void)
{
  SetupStore(SETUP_SLEEPTIME,   AtmoSetup.SleepTime);
  SetupStore(SETUP_PERCENTNEW,  AtmoSetup.PercentNew);
  SetupStore(SETUP_SATOFFSET,   AtmoSetup.SatOffset);
  SetupStore(SETUP_DARKLIMIT,   AtmoSetup.DarkLimit);
  SetupStore(SETUP_MODE,        AtmoSetup.Mode);
  SetupStore(SETUP_STATICRED,   AtmoSetup.StaticRed);
  SetupStore(SETUP_STATICGREEN, AtmoSetup.StaticGreen);
  SetupStore(SETUP_STATICBLUE,  AtmoSetup.StaticBlue);
  SetupStore(SETUP_CALIBRED,    AtmoSetup.CalibRed);
  SetupStore(SETUP_CALIBGREEN,  AtmoSetup.CalibGreen);
  SetupStore(SETUP_CALIBBLUE,   AtmoSetup.CalibBlue);
  SetupStore(SETUP_RAWRED,      AtmoSetup.RawRed);
  SetupStore(SETUP_RAWGREEN,    AtmoSetup.RawGreen);
  SetupStore(SETUP_RAWBLUE,     AtmoSetup.RawBlue);
}

eOSState cMenuSetupAtmo::ProcessKey( eKeys Key )
{
  int  prevMode = AtmoSetup.Mode;

  eOSState state = cOsdMenu::ProcessKey(Key);

  if( prevMode != AtmoSetup.Mode)
  {
    Set();
    state = osContinue;
  }

  if (state == osUnknown)
  {
    switch (Key)
    {
      case kOk: 
           Store();
           state = osBack;
      break;

      default:
      break;
    }
  }
  return state;
}
